/**********************************************************************
*          Copyright (c) 2007, Hogeschool voor de Kunsten Utrecht
*                      Hilversum, the Netherlands
*                          All rights reserved
***********************************************************************
*  This program is free software: you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation, either version 3 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program.
*  If not, see <http://www.gnu.org/licenses/>.
***********************************************************************
*
*  File name     : MulticastReceiver.cpp
*  System name   : Receiver for UDP multicast packets
* 
*  Description   : Implementation file
*
*
*  Author        : Marc_G
*  E-mail        : marcg@dinkum.nl
*
**********************************************************************/

#include <string>
#include <MulticastReceiver.h>


MulticastReceiver::MulticastReceiver(char* kernel_group,char* client_group,short port)
{
  // Open new UDP socket 
  if((s=socket(AF_INET,SOCK_DGRAM,0)) == -1) {
     throw(1);
  } // if

   // Allow multiple sockets to use the same port number
  int on=1;
  if(setsockopt(s,SOL_SOCKET,SO_REUSEADDR, (const char*)&on,sizeof(on)) == -1) {
      throw(2);
  } // if

  // Set up address struct
  memset(&addr,0,sizeof(addr));
  addr.sin_family=AF_INET;
  //addr.sin_addr.s_addr=htonl(INADDR_ANY); // any address  
  addr.sin_addr.s_addr = inet_addr(client_group); // specific group address
  addr.sin_port=htons(port);
  addrsize=sizeof(addr);
     
  /*
   * Tune in to the address(es) specified in addr
   */
  if(bind(s,(struct sockaddr *) &addr,sizeof(addr)) == -1) {
     throw(3);
  } // if
 
  /*
   * Request the kernel to join a multicast group
   */
  mreq.imr_multiaddr.s_addr=inet_addr(kernel_group);
  mreq.imr_interface.s_addr=htonl(INADDR_ANY);
  if (setsockopt(s,IPPROTO_IP,IP_ADD_MEMBERSHIP,&mreq,sizeof(mreq)) == -1) {
     throw(4);
  } // if

  recv_flags=0;
} // MulticastReceiver


MulticastReceiver::~MulticastReceiver()
{
  close(s); // close the socket
} // ~MulticastReceiver()


int MulticastReceiver::read(char* msgbuf,int bufsize)
{
  /*
   * In this function we'll read data directly into the given buffer.
   * The caller has to make sure this buffer has been allocated and
   * is large enough to append a zero if this is necessary.
   *
   */
  bytes_read=recvfrom(s,msgbuf,bufsize,recv_flags,
                        (struct sockaddr *) &addr,
                        (socklen_t *)&addrsize);
  if(bytes_read == -1) throw(5);

  return bytes_read;
} // MulticastReceiver::read()


int MulticastReceiver::read(std::string& msg)
{
  /*
   * In this function we'll create a string, which means we have to make
   *  sure that the char array ends with a zero. Therefore we'll limit
   *  the maximum number of bytes to be read by recvfrom() to 1 less than
   *  the receive buffer size so we can safely append a zero
   */
  bytes_read=recvfrom(s,receive_buffer,RECV_BUFSIZE-1,recv_flags,
                        (struct sockaddr *) &addr,
                        (socklen_t *)&addrsize);
  if(bytes_read == -1) throw(5);

  receive_buffer[bytes_read] = 0; // force string end
  msg=receive_buffer;

  return bytes_read;
} // MulticastReceiver::read()

